from wwtiotsdk.metainfo.parameter_lookup import ParameterLookup, PARAMETER_METADATA


def test_has():
    # Test that the has method correctly identifies existing and non-existing enum members
    assert ParameterLookup.has("powerVoltage") is True
    assert ParameterLookup.has("nonExistent") is False


def test_str():
    # Test that the string representation of enum members is correct
    assert str(ParameterLookup.VOLTAGE) == "powerVoltage"
    assert str(ParameterLookup.TEMPERATURE) == "temperature"


def test_repr():
    # Test that the repr representation of enum members is correct
    assert repr(ParameterLookup.VOLTAGE) == "powerVoltage"
    assert repr(ParameterLookup.TEMPERATURE) == "temperature"


def test_parameter_metadata():
    # Test that the parameter metadata is correctly defined
    assert PARAMETER_METADATA["powerVoltage"].long_name == "Voltage"
    assert PARAMETER_METADATA["powerVoltage"].unit == "V"
    assert PARAMETER_METADATA["temperature"].long_name == "Temperature"
    assert PARAMETER_METADATA["temperature"].unit == "°C"
    assert PARAMETER_METADATA["turbidity"].long_name == "Turbidity"
    assert PARAMETER_METADATA["turbidity"].unit == "NTU"
    assert PARAMETER_METADATA["trypTempCorr"].long_name == "Tryptophan Temp. Corrected"
    assert PARAMETER_METADATA["trypTempCorr"].unit == "ppb"
    assert PARAMETER_METADATA["cdomTempCorr"].long_name == "CDOM Temp. Corrected"
    assert PARAMETER_METADATA["cdomTempCorr"].unit == "µg/L"
    assert (
        PARAMETER_METADATA["electricConductivity"].long_name == "Electric Conductivity"
    )
    assert PARAMETER_METADATA["electricConductivity"].unit == "µS/cm"
    assert PARAMETER_METADATA["totalColi"].long_name == "Total Coli"
    assert PARAMETER_METADATA["totalColi"].unit == "CFU/100mL"
    assert PARAMETER_METADATA["faecalColi"].long_name == "Faecal Coli"
    assert PARAMETER_METADATA["faecalColi"].unit == "CFU/100mL"
    assert PARAMETER_METADATA["eColi"].long_name == "E Coli"
    assert PARAMETER_METADATA["eColi"].unit == "CFU/100mL"
    assert PARAMETER_METADATA["batteryLevel"].long_name == "Battery Level"
    assert PARAMETER_METADATA["batteryLevel"].unit == "%"
    assert PARAMETER_METADATA["oxygenSat"].long_name == "Oxygen Saturation"
    assert PARAMETER_METADATA["oxygenSat"].unit == "%"
    assert PARAMETER_METADATA["oxygenCon"].long_name == "Oxygen Concentration"
    assert PARAMETER_METADATA["oxygenCon"].unit == "mg/L"
    assert PARAMETER_METADATA["depth"].long_name == "Depth"
    assert PARAMETER_METADATA["depth"].unit == "m"
    assert (
        PARAMETER_METADATA["chlorophyllConcentration"].long_name
        == "Chlorophyll Concentration"
    )
    assert PARAMETER_METADATA["chlorophyllConcentration"].unit == "µg/L"
    assert (
        PARAMETER_METADATA["phycocyaninConcentration"].long_name
        == "Phycocyanin Concentration"
    )
    assert PARAMETER_METADATA["phycocyaninConcentration"].unit == "µg/L"
    assert (
        PARAMETER_METADATA["chlorophyllFluorescence"].long_name
        == "Chlorophyll Fluorescence"
    )
    assert PARAMETER_METADATA["chlorophyllFluorescence"].unit == "RFU"
    assert (
        PARAMETER_METADATA["phycocyaninFluorescence"].long_name
        == "Phycocyanin Fluorescence"
    )
    assert PARAMETER_METADATA["phycocyaninFluorescence"].unit == "RFU"



from datetime import datetime, timezone
from wwtiotsdk.influxdb.dataframe_base import DataframeBase
from wwtiotsdk.metainfo.parameter_lookup import ParameterLookup


class DataframeRecords2DTime(DataframeBase):
    """
    Class to transform data into a 2D time record based format, that can be ingested into InfluxDB.
    """

    def __init__(self, source: str, device: str, measurement: str):
        """
        Initialize the DataframeRecords2DTime object.

        args:
            source (str): The source of the data.
            device (str): The device name.
            measurement (str): The measurement name.
        """
        self.source = source
        self.device = device
        self.measurement = measurement
        self.datapoints: list[dict] = []
        self._validate()

    def _validate(self) -> None:
        """
        Validate the source, device, and measurement attributes.
        """
        if not self.source or not self.device or not self.measurement:
            raise ValueError("Source, device, and measurement must be provided.")
        if (
            not isinstance(self.source, str)
            or not isinstance(self.device, str)
            or not isinstance(self.measurement, str)
        ):
            raise TypeError("Source, device, and measurement must be strings.")
        if len(self.source) == 0 or len(self.device) == 0 or len(self.measurement) == 0:
            raise ValueError("Source, device, and measurement cannot be empty strings.")

    def add_datapoint(self, time: datetime, parameter: str, value: float | int) -> None:
        """
        Add a datapoint to the 2D time record.

        args:
            time (datetime): The time of the record.
            parameter (str): The parameter name.
            value (float|int): The value of the parameter.
        """

        if time.tzinfo != timezone.utc:
            raise ValueError("The 'time' parameter must have a UTC timezone.")

        unix_ms = int(time.timestamp() * 1000)

        if not ParameterLookup.has(parameter):
            raise ValueError(f"Parameter '{parameter}' is not valid.")

        if not isinstance(value, (int, float)):
            raise TypeError(
                f"Value for parameter '{parameter}' must be an int or float."
            )

        value = float(value)

        self.datapoints.append(
            {"time": unix_ms, "parameter": parameter, "value": value}
        )

    def get_influx_records(self, valid: bool = True) -> list[dict]:
        """
        Get the InfluxDB records from the 2D time record.

        returns:
            list[dict]: The InfluxDB records.
        """
        return [
            {
                "measurement": self.measurement,
                "tags": {
                    "source": self.source,
                    "device": self.device,
                    "parameter": datapoint["parameter"],
                },
                "time": datapoint["time"],
                "fields": {"value": datapoint["value"], "valid": valid},
            }
            for datapoint in self.datapoints
        ]

    def get_precision(self) -> str:
        """
        Get the precision of the records."
        """
        return "ms"



from wwtiotsdk.influxdb.connector import InfluxDBConnector
import pytest


def test_raise_exception(mocker):

    with pytest.raises(Exception) as excinfo:

        client_mock = mocker.Mock()
        client_mock.write = mocker.Mock(side_effect=Exception("Write error"))

        with InfluxDBConnector("", "", "") as client:
            client.client = client_mock
            client.write(mocker.Mock())

    assert str(excinfo.value) == "Write error"



from enum import Enum
from dataclasses import dataclass


@dataclass
class ParInfo:
    long_name: str
    unit: str


class AutoStringEnum(str, Enum):
    """
    A base Enum class that automatically casts its members to strings.
    Inherits from both `str` and `Enum`.
    """

    def __new__(cls, value, *args, **kwargs):
        obj = str.__new__(cls, value)
        obj._value_ = value
        return obj

    def __str__(self):
        return self.value

    def __repr__(self):
        return self.value


class ParameterLookup(AutoStringEnum):
    VOLTAGE = "powerVoltage"
    TEMPERATURE = "temperature"
    TURBIDITY = "turbidity"
    TRYP_TEMP_CORR = "trypTempCorr"
    CDOM_TEMP_CORR = "cdomTempCorr"
    ELECTRIC_CONDUCTIVITY = "electricConductivity"
    TOTAL_COLI = "totalColi"
    FAECAL_COLI = "faecalColi"
    E_COLI = "eColi"
    BATTERY_LEVEL = "batteryLevel"
    OXYGEN_SAT = "oxygenSat"
    OXYGEN_CON = "oxygenCon"
    DEPTH = "depth"
    CHLA_CON = "chlorophyllConcentration"
    CHLA_FLOUR = "chlorophyllFluorescence"
    PHYCO_CON = "phycocyaninConcentration"
    PHYCO_FLOUR = "phycocyaninFluorescence"

    def __str__(self):
        return self.value

    @classmethod
    def has(cls, value: str) -> bool:
        """
        Check if the given value exists in the enum.
        :param value: The value to check.
        :return: True if the value exists, False otherwise.
        """
        return value in cls._value2member_map_


# Metadata for parameters
PARAMETER_METADATA = {
    ParameterLookup.VOLTAGE: ParInfo("Voltage", "V"),
    ParameterLookup.TEMPERATURE: ParInfo("Temperature", "°C"),
    ParameterLookup.TURBIDITY: ParInfo("Turbidity", "NTU"),
    ParameterLookup.TRYP_TEMP_CORR: ParInfo("Tryptophan Temp. Corrected", "ppb"),
    ParameterLookup.CDOM_TEMP_CORR: ParInfo("CDOM Temp. Corrected", "µg/L"),
    ParameterLookup.ELECTRIC_CONDUCTIVITY: ParInfo("Electric Conductivity", "µS/cm"),
    ParameterLookup.TOTAL_COLI: ParInfo("Total Coli", "CFU/100mL"),
    ParameterLookup.FAECAL_COLI: ParInfo("Faecal Coli", "CFU/100mL"),
    ParameterLookup.E_COLI: ParInfo("E Coli", "CFU/100mL"),
    ParameterLookup.BATTERY_LEVEL: ParInfo("Battery Level", "%"),
    ParameterLookup.OXYGEN_SAT: ParInfo("Oxygen Saturation", "%"),
    ParameterLookup.OXYGEN_CON: ParInfo("Oxygen Concentration", "mg/L"),
    ParameterLookup.DEPTH: ParInfo("Depth", "m"),
    ParameterLookup.CHLA_CON: ParInfo("Chlorophyll Concentration", "µg/L"),
    ParameterLookup.CHLA_FLOUR: ParInfo("Chlorophyll Fluorescence", "RFU"),
    ParameterLookup.PHYCO_CON: ParInfo("Phycocyanin Concentration", "µg/L"),
    ParameterLookup.PHYCO_FLOUR: ParInfo("Phycocyanin Fluorescence", "RFU"),
}



class DataframeBase:

    def get_influx_records():
        raise NotImplementedError("This method should be implemented in subclasses.")

    def get_precision():
        raise NotImplementedError("This method should be implemented in subclasses.")



from wwtiotsdk.influxdb.dataframe_records_2d_time import DataframeRecords2DTime
from datetime import datetime, timezone
import pytest


def test_validate():
    # Test valid input
    obj = DataframeRecords2DTime("source", "device", "measurement")

    # Test invalid input
    with pytest.raises(ValueError):
        obj = DataframeRecords2DTime("", "device", "measurement")

    with pytest.raises(TypeError):
        obj = DataframeRecords2DTime(123, "device", "measurement")

    with pytest.raises(ValueError):
        obj = DataframeRecords2DTime("source", "", "measurement")

    with pytest.raises(ValueError):
        obj = DataframeRecords2DTime("source", "device", None)


def test_add_datapoint():
    obj = DataframeRecords2DTime("source", "device", "measurement")

    # Test valid datapoint
    obj.add_datapoint(
        datetime(2023, 10, 1, 12, 0, tzinfo=timezone.utc), "temperature", 25.5
    )
    assert len(obj.datapoints) == 1
    assert obj.datapoints[0]["time"] == 1696161600000
    assert obj.datapoints[0]["parameter"] == "temperature"
    assert obj.datapoints[0]["value"] == 25.5

    obj.add_datapoint(
        datetime(2023, 10, 1, 12, 0, tzinfo=timezone.utc), "batteryLevel", 100
    )
    assert obj.datapoints[1]["time"] == 1696161600000
    assert obj.datapoints[1]["parameter"] == "batteryLevel"
    assert obj.datapoints[1]["value"] == 100.0

    # Test invalid parameter
    with pytest.raises(ValueError):
        obj.add_datapoint(
            datetime(2023, 10, 1, 12, 0, tzinfo=timezone.utc), "invalid_param", 25.5
        )

    # Test invalid value type
    with pytest.raises(TypeError):
        obj.add_datapoint(
            datetime(2023, 10, 1, 12, 0, tzinfo=timezone.utc),
            "temperature",
            "invalid_value",
        )


def test_get_influx_records():
    obj = DataframeRecords2DTime("source", "device", "measurement")
    obj.add_datapoint(
        datetime(2023, 10, 1, 12, 0, tzinfo=timezone.utc), "temperature", 25.5
    )
    obj.add_datapoint(
        datetime(2023, 10, 1, 12, 1, tzinfo=timezone.utc), "turbidity", 60.0
    )

    records = obj.get_influx_records(valid=True)
    assert len(records) == 2
    r0 = records[0]
    assert r0["time"] == 1696161600000
    assert r0["fields"]["valid"] == True
    assert r0["fields"]["value"] == 25.5
    assert r0["tags"]["source"] == "source"
    assert r0["tags"]["device"] == "device"
    assert r0["tags"]["parameter"] == "temperature"
    assert r0["measurement"] == "measurement"

    r1 = records[1]
    assert r1["time"] == 1696161660000
    assert r1["fields"]["valid"] == True
    assert r1["fields"]["value"] == 60.0
    assert r1["tags"]["source"] == "source"
    assert r1["tags"]["device"] == "device"
    assert r1["tags"]["parameter"] == "turbidity"
    assert r1["measurement"] == "measurement"

    # Test invalid timezone
    with pytest.raises(ValueError):
        obj.add_datapoint(datetime(2023, 10, 1, 12, 0), "temperature", 25.5)


def test_get_precision():
    obj = DataframeRecords2DTime("source", "device", "measurement")
    precision = obj.get_precision()
    assert precision == "ms"



import boto3
import json
from influxdb_client_3 import InfluxDBClient3, flight_client_options
import certifi

from wwtiotsdk.influxdb.dataframe_base import DataframeBase


class InfluxDBConnector:

    def __init__(self, host: str, database: str, token: str):
        self.host = host
        self.database = database
        self.token = token
        self.client = None

    def connect(self):
        """Create the InfluxDB client"""

        fh = open(certifi.where(), "r")
        cert = fh.read()
        fh.close()

        self.client = InfluxDBClient3(
            host=self.host,
            token=self.token,
            database=self.database,
            flight_client_options=flight_client_options(tls_root_certs=cert),
        )

    def close(self):
        """Close the InfluxDB client"""
        if self.client:
            self.client.close()
            self.client = None

    def query(self, query: str) -> list[dict]:
        """Execute a query and return the result"""
        if not self.client:
            raise Exception("Client is not connected. Please call connect() first.")

        table = self.client.query(query, language="sql", mode="all")
        return table.to_pylist()

    def write(self, dataframe: DataframeBase) -> None:
        """Write records to InfluxDB"""
        if not self.client:
            raise Exception("Client is not connected. Please call connect() first.")

        precision = dataframe.get_precision()
        records = dataframe.get_influx_records()

        self.client.write(
            record=records,
            write_precision=precision,
        )

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, _exc_value, _traceback):
        if exc_type is not None:
            return False
        self.close()
        return True


def get_secret_from_aws(name: str) -> str:
    try:
        client = boto3.client("secretsmanager")
        res = client.get_secret_value(SecretId=name)
        return json.loads(res["SecretString"])["token"]
    except Exception as err:
        raise ValueError(
            f"Error occurred while fetching '{name}'secret from AWS Secrets Manager: {str(err)}"
        )
